@@ -6,11 +6,12 @@ This just wraps the normal mongrel2 stuff to make it easier to work with websock
 from uuid import uuid4
 import logging
 from struct import pack # for network byte order
+import time
 
 # third party
 from mongrel2 import handler, request
 
-__version__ = '0.1.9'
+__version__ = '0.2'
 
 logger = logging.getLogger(__name__)
 
@@ -71,7 +72,7 @@ class Request(request.Request):
         try:
             return self.body.decode('utf-8')
 
-        except UnicodeError, e:
+        except UnicodeError as e:
             raise WebsocketError(CLOSE_INVALID_FRAME_DATA, 'invalid UTF')
 
     def __init__(self, req):
@@ -153,12 +154,15 @@ class Request(request.Request):
 
 
 class Connection(handler.Connection):
+    """A replacement for m2's vanilla Connection class, use this one instead to make
+    working with websockets easy-peasy"""
+
+    aborted_timeout = 5.0
+    """how many seconds before aborted connections should be cleaned up"""
 
     request_class = Request
     """the default request class"""
 
-    """A replacement for m2's vanilla Connection class, use this one instead to make
-    working with websockets easy-peasy"""
     def __init__(self, sub_addr, pub_addr):
         sender_id = uuid4().hex
         super(Connection, self).__init__(sender_id, sub_addr, pub_addr)
@@ -182,7 +186,12 @@ class Connection(handler.Connection):
 
                 elif ws_req.is_disconnect():
                     # drop these through so they can be handled
-                    logger.debug('{} DISCONNECT'.format(ws_req.conn_id))
+                    logger.info('{} DISCONNECT'.format(ws_req.conn_id))
+                    if self.is_aborted(ws_req):
+                        logger.debug('{} ABORTED, ignoring disconnect'.format(ws_req.conn_id))
+                        self.remove_aborted(ws_req)
+                        continue
+
                     ws_req.needs_response = False
 
                 elif not ws_req.is_websocket():
@@ -190,22 +199,22 @@ class Connection(handler.Connection):
                     ws_req.needs_response = True
 
                 elif ws_req.is_close():
-                    if self.is_closed(ws_req):
-                        logger.info('{} CLOSE HANDSHAKE COMPLETE'.format(ws_req.conn_id))
+                    if self.is_aborted(ws_req):
+                        logger.debug('{} ABORTED HANDSHAKE COMPLETE'.format(ws_req.conn_id))
                         # if the connection id is in aborted, then the close handshake is complete,
                         # we've sent down an OP_CLOSE and received one back
-                        del self.aborted[ws_req.conn_id]
+                        self.remove_aborted(ws_req)
+                        self.close(ws_req)
                         continue
 
-                    else:
-                        # we still want to return this request to give the controller a chance to do other cleanup
-                        # even though we have done everything we needed to do to close the connection already
-                        self.reply_close(ws_req)
-                        ws_req.needs_response = False
+                    # we still want to return this request to give the controller a chance to do other cleanup
+                    # even though we have done everything we needed to do to close the connection already
+                    self.close_websocket(ws_req)
+                    ws_req.needs_response = False
 
                 else:
-                    if self.is_closed(ws_req):
-                        logger.info('{} CLOSED, ignoring opcode {}'.format(ws_req.conn_id, ws_req.opcode))
+                    if self.is_aborted(ws_req):
+                        logger.debug('{} ABORTED, ignoring opcode {}'.format(ws_req.conn_id, ws_req.opcode))
                         continue
 
                     elif ws_req.is_ping():
@@ -229,7 +238,7 @@ class Connection(handler.Connection):
 
                 break
 
-            except Exception, e:
+            except Exception as e:
                 self.abort(m2_req, e) # we use m2_req since the ws_req creation can fail
                 continue
 
@@ -288,12 +297,11 @@ class Connection(handler.Connection):
         logger.info('{} PING -> replying PONG'.format(req.conn_id))
         self.reply_websocket(req, req.body, OP_PONG)
 
-    def reply_close(self, req):
+    def close_websocket(self, req):
         """reply to a close request"""
         logger.info('{} CLIENT CLOSED'.format(req.conn_id))
         # the client has closed on us, we need to send down OP_CLOSE to complete the handshake
         self.reply_websocket(req, req.body, OP_CLOSE)
-
         self.close(req)
 
     def reply_websocket(self, req, *args, **kwargs):
@@ -307,7 +315,7 @@ class Connection(handler.Connection):
         this will save the connection and send the OP_CLOSE opcode to the client, if/when
         we get an OP_CLOSE from the client, we will delete the connection
         """
-        if self.is_closed(req):
+        if self.is_aborted(req):
             logger.info('{} already aborted'.format(req.conn_id))
             return
 
@@ -325,13 +333,37 @@ class Connection(handler.Connection):
         else:
             ews = WebsocketError(CLOSE_NORMAL, 'Normal Abort')
             body = str(ews)
-
             logger.info('{} NORMAL ABORT'.format(req.conn_id))
 
         self.reply_websocket(req, body, opcode=OP_CLOSE)
-        self.aborted[req.conn_id] = (e, req.sender)
+        self.add_aborted(req)
 
-    def is_closed(self, req):
+    def is_aborted(self, req):
         """return True if this connection has been closed previously"""
         return req.conn_id in self.aborted
 
+    def add_aborted(self, req):
+        self.aborted[req.conn_id] = (time.time(), req.sender)
+
+    def remove_aborted(self, req):
+        del self.aborted[req.conn_id]
+
+    def cleanup_aborted(self):
+        """this will cleanup the aborted connection cache, this was originally
+        run right before waiting for another request, but I've instead decided
+        to let it be manually done, which will make it easier when you have multiple
+        handlers, so they aren't all wasting their time cleaning up connections.
+        I've also moved all abort handling code behind functions so it can easily
+        be overriden, like, for instance, if you wanted to store them in redis because
+        you have tons of handlers"""
+        logger.info('CLEANING UP ABORTED CONNECTIONS')
+        now = time.time()
+        for conn_id, (t, sender) in self.aborted.items():
+            if now > t + self.aborted_timeout:
+                logger.debug('cleaning up aborted connection {}'.format(conn_id))
+                self.send(sender, conn_id, '')
+
+        logger.debug('DONE CLEANING UP ABORTED CONNECTIONS')
+
+
+
